#!/usr/bin/env python
# -*- coding: cp949 -*-

# GetNameFromOMR.py: construct Korean name from OMR input
#
# Copyright (c) 2007-2008 Homin Lee <ff4500@gmail.com>
# License: LGPL (see lgpl.txt)
#
# THIS PROGRAM COMES WITH NO WARRANT

import os
import hangul
from EasyExcel import *

def _getChosungBy(idx):
    ''' ʼ ε(idx)  ڵ  return
    >>> _getChosungBy(0)
    u'\u3131'
    '''
    return hangul.Jaeum.Choseong[idx]

def _getJungseongBy(idx):
    ''' ߼ ε(idx)  ڵ  return
    >>> _getJungseongBy(0)
    u'\u314f'
    '''
    return hangul.Moeum.Jungseong[idx]

def _getJongseongBy(idx):
    '''  ε(idx)  ڵ  return
    >>> _getJongseongBy(0)
    u''
    '''
    return hangul.Jaeum.Jongseong[idx]

def _getKorChar(charIdx):
    ''' ʼ ߼  ε (fIdx, mIdx, lIdx) 
    ڵ ѱ return  
    >>> print _getKorChar((18, 8, 21)).encode('cp949')
    ȫ
    >>> print _getKorChar((0, 20, 8)).encode('cp949')
    
    >>> print _getKorChar((3, 8, 21)).encode('cp949')
    
    '''
    fIdx, mIdx, lIdx = charIdx
    return hangul.join((_getChosungBy(fIdx),
                       _getJungseongBy(mIdx),
                       _getJongseongBy(lIdx)))

def _readExcelOMR(xls, rowNo):
    ''' ڵ xls  rowNo  OMR ε  о
    >>> _readExcelOMR(EasyExcel(os.path.realpath('̸б.xls')), 3)
    ([18, 8, 21], [0, 20, 8], [3, 8, 21])
    '''
    _c = xls.getCell
    return (\
        map(int, (_c(1, rowNo, 1), _c(1, rowNo, 2), _c(1, rowNo, 3))),
        map(int, (_c(1, rowNo, 4), _c(1, rowNo, 5), _c(1, rowNo, 6))),
        map(int, (_c(1, rowNo, 7), _c(1, rowNo, 8), _c(1, rowNo, 9)))
    )

def _docTest():
    import doctest
    doctest.testmod()

def _test():
    xlsPath = os.path.realpath('̸б.xls')
    print '  : %s'%xlsPath
    xls = EasyExcel(xlsPath)
    
    print '׽Ʈ (3) '
    cIdx1, cIdx2, cIdx3 = _readExcelOMR(xls, 3)
    
    print 'ù°  =',
    nameChar1 = _getKorChar(cIdx1)
    print nameChar1
    
    print 'ι°  =',
    nameChar2 = _getKorChar(cIdx2)
    print nameChar2
    
    print '°  =',
    nameChar3 = _getKorChar(cIdx3)
    print nameChar3

    print ' ̸ '
    xls.setCell(1, 3, 10, nameChar1)
    xls.setCell(1, 3, 11, nameChar2)
    xls.setCell(1, 3, 12, nameChar3)
    xls.setCell(1, 3, 13, ''.join((nameChar1, nameChar2, nameChar3)))

    newXlsPath = xlsPath.replace('.xls','_test.xls')
    print ' ̸  : %s'%newXlsPath
    xls.save(newXlsPath)
    print ' ݱ'
    xls.close()

if __name__ == '__main__':
    #_docTest()
    _test()

